using System.Diagnostics.Metrics;

namespace Frontend.Metrics;

public class Erc20TokenMetrics
{
    private readonly IMeterFactory _meterFactory;
    private readonly TokenClient _tokenClient;
    private ObservableGauge<double>? _totalSupplyGauge;

    public Erc20TokenMetrics(IMeterFactory meterFactory, TokenClient tokenClient)
    {
        _meterFactory = meterFactory;
        _tokenClient = tokenClient;
    }

    public void CreateMetrics()
    {
        var meter = _meterFactory.Create("RoestiCoin");
        _totalSupplyGauge = meter.CreateObservableGauge("RoestiCoin.Supply.Total", () => GetTotalSupply(), unit: "ROC");
    }

    private double GetTotalSupply()
    {
        var totalSupply = _tokenClient.TotalSupply();
        totalSupply.Wait();
        return (double)totalSupply.Result;
    }
}