using System.Runtime.CompilerServices;
using Frontend;
using Frontend.BackgroundServices;
using Frontend.Configuration;
using Frontend.Metrics;
using Frontend.Services;
using Nethereum.Web3;
using Nethereum.Web3.Accounts;
using OpenTelemetry.Metrics;
using OpenTelemetry.Resources;
using Org.BouncyCastle.Utilities;

// the location differ in dev and releas mode.
var tokenAbiLocation = Environment.GetEnvironmentVariable("TOKEN_ABI_LOCATION")!;
var tokenAbi = File.ReadAllText(tokenAbiLocation)!;

var chainApiUrl = Environment.GetEnvironmentVariable("API_URL")!;
var accountPrivateKey = Environment.GetEnvironmentVariable("ACCOUNT_PRIVATE_KEY")!;


var builder = WebApplication.CreateBuilder(args);

// Configurations
var chainConfiguration = builder.Configuration.GetSection("Chain");
var chainId = int.Parse(chainConfiguration["ChainId"] ?? "11155111"); // fallback to Sepolia 
var chainSettings = new ChainSettings();
chainConfiguration.Bind(chainSettings);

// Add services to the container.
builder.Services.AddHostedService<PrometheusService>();
builder.Services.AddSingleton<Erc20TokenMetrics>();
builder.Services.AddSingleton<EthMetrics>();

Account account = new Account(
    accountPrivateKey,
    chainId: chainId
);

builder.Services.AddSingleton(provider => chainSettings);

builder.Services.AddSingleton(provider => new TokenClient(
    chainApiUrl,
    chainConfiguration["TokenAddress"]!,
    tokenAbi,
    accountPrivateKey,
    chainId
));

builder.Services.AddSingleton(provider=>account);

builder.Services.AddSingleton(provider => new Web3(account, chainApiUrl));

builder.Services.AddSingleton<PoolMetrics>();



builder.Services.AddRazorPages();
builder.Services.AddOpenTelemetry()
    .WithMetrics(builder =>
    {
        builder.AddPrometheusExporter();

        builder.AddMeter("Microsoft.AspNetCore.Hosting",
                         "Microsoft.AspNetCore.Server.Kestrel",
                         "RoestiCoin");
    });

var app = builder.Build();
// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Error");
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();

    app.UseHttpsRedirection();
}

app.MapPrometheusScrapingEndpoint();
app.UseStaticFiles();

app.UseRouting();

app.UseAuthorization();

app.MapRazorPages();

app.Run();
